#pragma once

// So, windows.h and stdint.h can be somewhat redundant, I wish I did not
// 	need to include both. But, I want reliable sized integer types, and 
// 	I don't want windows.h to be exposed over the entire project, so
// 	this is what I chose to do.
#include <stdint.h>

#define i8 int8_t 
#define u8 uint8_t 
#define i16 int16_t 
#define u16 uint16_t 
#define i32 int32_t 
#define u32 uint32_t 
#define i64 int64_t 
#define u64 uint64_t 
#define i16_l int_least16_t 
#define u16_l uint_least16_t 
#define i32_l int_least32_t 
#define u32_l uint_least32_t 
#define i64_l int_least64_t 
#define u64_l uint_least64_t 
#define i8_f int_fast8_t 
#define u8_f uint_fast8_t 
#define i16_f int_fast16_t 
#define u16_f uint_fast16_t 
#define i32_f int_fast32_t 
#define u32_f uint_fast32_t 
#define i64_f int_fast64_t 
#define u64_f uint_fast64_t 
#define iptr intptr_t 
#define uptr uintptr_t 
#define r32 float
#define r64 double

#define TYPE_MIN(a) _Generic((a),\
	i8: INT8_MIN,\
	u8: 0,\
	i16: INT16_MIN,\
	u16: 0,\
	i32: INT32_MIN,\
	u32: 0,\
	i64: INT64_MIN,\
	u64: 0,\
	i16_l: INT_LEAST16_MIN,\
	u16_l: 0,\
	i32_l: INT_LEAST32_MIN,\
	u32_l: 0,\
	i64_l: INT_LEAST64_MIN,\
	u64_l: 0,\
	i8_f: INT_FAST8_MIN,\
	u8_f: 0,\
	i16_f: INT_FAST16_MIN,\
	u16_f: 0,\
	i32_f: INT_FAST32_MIN,\
	u32_f: 0,\
	i64_f: INT_FAST64_MIN,\
	u64_f: 0)


#define TYPE_MAX(a) _Generic((a),\
	i8: INT8_MAX,\
	u8: UINT8_MAX,\
	i16: INT16_MAX,\
	u16: UINT16_MAX,\
	i32: INT32_MAX,\
	u32: UINT32_MAX,\
	i64: INT64_MAX,\
	u64: UINT64_MAX,\
	i16_l: INT_LEAST16_MAX,\
	u16_l: UINT_LEAST16_MAX,\
	i32_l: INT_LEAST32_MAX,\
	u32_l: UINT_LEAST32_MAX,\
	i64_l: INT_LEAST64_MAX,\
	u64_l: UINT_LEAST64_MAX,\
	i8_f: INT_FAST8_MAX,\
	u8_f: UINT_FAST8_MAX,\
	i16_f: INT_FAST16_MAX,\
	u16_f: UINT_FAST16_MAX,\
	i32_f: INT_FAST32_MAX,\
	u32_f: UINT_FAST32_MAX,\
	i64_f: INT_FAST64_MAX,\
	u64_f: UINT_FAST64_MAX)

struct PlatformAllocInfo
{
	char *name;
	u64 id;
	struct PlatformAllocInfo *next;
	size_t size;
};

// function signiture for memory facilities
typedef void *Allocator(size_t, char*);
typedef void DeAllocator(void *);
typedef void *ReAllocator(void*, size_t, char*);

#ifndef __MEMORYPROVIDER
#define __MEMORYPROVIDER
struct MemoryProvider
{
	Allocator *allocator;
	ReAllocator *reallocator;
	DeAllocator *deallocator;
};
#endif

void *PlatformStartPerfClock();
r32 PlatfromEndPerfClock(void*);

extern struct MemoryProvider platformMemory;
extern struct MemoryProvider tempMem;

void LogSetOnOff(int onOff);
void Log(char *string);
void LogFormat(char *string, ...);
