// *Really* minimal PCG32 code / (c) 2014 M.E. O'Neill / pcg-random.org
// Licensed under Apache License 2.0 (NO WARRANTY, etc. see website)


/* Modified and adapted by Andrew Dunetz (2019)
 *
 * */

typedef struct { unsigned long long state; unsigned long long  inc; } pcg32_random_t;
// This is a default seed, I should really write a seed function that makes this a 
// dynamic value. In the meantime, this is an acceptable seed. For debug reasons,
// it is nice to keep the seed the same early in development so errors are duplicable.
pcg32_random_t pcg = (pcg32_random_t){
	.state=0b10011111111100011010100110111011,
	.inc =0b00100001011001101101001000100110
};

char alphanum[] = " qwertyuiopasdfghjklzxcvbnm1234567890QWERTYUIOPASDFGHJKLZXCVBNM";

unsigned int pcg32_random_r(pcg32_random_t* rng)
{
    unsigned long long oldstate = rng->state;
    // Advance internal state
    rng->state = oldstate * 6364136223846793005ULL + (rng->inc|1);
    // Calculate output function (XSH RR), uses old state for max ILP
    unsigned int xorshifted = ((oldstate >> 18u) ^ oldstate) >> 27u;
    unsigned int rot = oldstate >> 59u;
    return (xorshifted >> rot) | (xorshifted << ((-rot) & 31));
}


unsigned int RandomRangeU(unsigned int min, unsigned int max)
{
	unsigned int range = max - min;
	return min + (pcg32_random_r(&pcg))%range;
}

unsigned int RandomU()
{ return pcg32_random_r(&pcg); }

int RandomI()
{ 	
	unsigned int u = RandomU();
	return *(int*)&u;
}

int RandomRangeI(int min, int max)
{
	unsigned int range = max - min;
	return min + RandomRangeU(0,range);
}

float RandomR()
{
	unsigned int val = pcg32_random_r(&pcg);
	float valf = (float)val;
	float max = (float)4294967296;
	return valf/max;
}

float RandomRangeR(float min, float max)
{
	float t = RandomR();
	return (1.0f-t)*min + t*max;
}

#define RandomMax(max) _Generic((max),\
	unsigned int: RandomRangeU,\
	int: RandomRangeI,\
	float: RandomRangeR)(0,max)

#define RandomRange(min,max) _Generic((min),\
	unsigned int: RandomRangeU,\
	int: RandomRangeI,\
	float: RandomRangeR)(min,max)

char RandomCharacter()
{
	int len = sizeof(alphanum) / sizeof(char);
	return alphanum[RandomMax(len)];
}

void FillString(char *out_string, size_t len)
{
	while (--len >=0)
	{ out_string[len] = RandomCharacter(); }
}

int Die(int sides)
{ return 1 + RandomMax(sides); }

int Dice(int* out, int num, int sides)
{
	int total = 0;
	for (int i = 0; i < num; i++)
	{
		int d = Die(sides);
		total += d;
		if (out)
		{ out[i] = d; }
	}
	return total;
}
